@@ -1,17 +1,13 @@
 """Declaration of base types for dali commands and their responses."""
 
-from __future__ import unicode_literals
-from __future__ import division
-from __future__ import absolute_import
 from dali import address
 from dali import frame
-from dali.compat import add_metaclass
-from dali.compat import python_2_unicode_compatible
 from dali.exceptions import MissingResponse
 from dali.exceptions import ResponseError
+import warnings
 
 
-class CommandTracker(type):
+class _CommandTracker(type):
     """Metaclass keeping track of all the types of Command we understand.
 
     Commands that have names starting with '_' are treated as abstract
@@ -25,17 +21,17 @@ class CommandTracker(type):
         else:
             if cls.__name__[0] != '_':
                 cls._commands.append(cls)
-
-    @classmethod
-    def commands(cls):
-        """
-        :return: List of known commands if there's any
-        """
-        return cls._commands
+        if not hasattr(cls, '_supported_devicetypes'):
+            cls._supported_devicetypes = set()
+        else:
+            if cls.devicetype != 0:
+                cls._supported_devicetypes.add(cls.devicetype)
+        for c in bases:
+            if issubclass(c, Command):
+                c._register_subclass(cls)
 
 
-@python_2_unicode_compatible
-class Response(object):
+class Response:
     """Some DALI commands cause a response from the addressed devices.
 
     The response is either an 8-bit backward frame encoding 8-bit data
@@ -55,6 +51,10 @@ class Response(object):
             raise TypeError("Response must be passed None or a BackwardFrame")
         self._value = val
 
+    @property
+    def raw_value(self):
+        return self._value
+
     @property
     def value(self):
         if self._value is None and self._expected:
@@ -69,6 +69,24 @@ class Response(object):
         except MissingResponse or ResponseError as e:
             return "{}".format(e)
 
+class NumericResponse(Response):
+    _expected = True
+
+    @property
+    def value(self):
+        if self._value is None:
+            return "(missing)"
+        if self._value and self._value.error:
+            return "(framing error)"
+        return self._value.as_integer
+
+class NumericResponseMask(NumericResponse):
+    @property
+    def value(self):
+        v = super(NumericResponseMask, self).value
+        if v == 255:
+            return "MASK"
+        return v
 
 class YesNoResponse(Response):
     _error_acceptable = True
@@ -93,9 +111,7 @@ class BitmapResponseBitDict(type):
             cls._bit_properties = bd
 
 
-@python_2_unicode_compatible
-@add_metaclass(BitmapResponseBitDict)
-class BitmapResponse(Response):
+class BitmapResponse(Response, metaclass=BitmapResponseBitDict):
     """A response that consists of several named bits.
 
     Bits are listed in subclasses with the least-sigificant bit first.
@@ -140,9 +156,7 @@ class BitmapResponse(Response):
             return "{}".format(e)
 
 
-@python_2_unicode_compatible
-@add_metaclass(CommandTracker)
-class Command(object):
+class Command(metaclass=_CommandTracker):
     """A command frame.
 
     Subclasses must provide a class method "from_frame" which, when
@@ -157,24 +171,43 @@ class Command(object):
     # 16 of IEC 62386-102 and tables 21 and 22 of IEC 62386-103.
     # Override them in subclasses if there is a tick in the
     # appropriate column.
-    _appctrl = False
-    _inputdev = False
-    _uses_dtr0 = False
-    _uses_dtr1 = False
-    _uses_dtr2 = False
-    _response = None
-    _sendtwice = False
+    #
+    # "response" is None if no response is expected, or a Response
+    # class to process the response.
+    appctrl = False
+    inputdev = False
+    uses_dtr0 = False
+    uses_dtr1 = False
+    uses_dtr2 = False
+    response = None
+    sendtwice = False
 
     # 16-bit frames may be interpreted differently if they are
     # preceded by the EnableDeviceType command.  If a command needs
-    # EnableDeviceType(foo) to be sent first, override _devicetype to
+    # EnableDeviceType(foo) to be sent first, override devicetype to
     # foo.  This parameter is ignored for all other frame lengths.
-    _devicetype = 0
+    devicetype = 0
+
+    # devicetype used to be called "_devicetype".  This property is
+    # here for compatibility with older code that relied on this.  It
+    # will be removed in a future release.
+    @property
+    def _devicetype(self):
+        warnings.warn("'_devicetype' has been renamed to 'devicetype'",
+                      DeprecationWarning, stacklevel=2)
+        return self.devicetype
 
     def __init__(self, f):
         assert isinstance(f, frame.ForwardFrame)
         self._data = f
 
+    _framesizes = {}
+
+    @classmethod
+    def _register_subclass(cls, subclass):
+        subs = cls._framesizes.setdefault(subclass._framesize, list())
+        subs.append(subclass)
+
     @classmethod
     def from_frame(cls, f, devicetype=0):
         """Return a Command instance corresponding to the supplied frame.
@@ -190,18 +223,17 @@ class Command(object):
         frame.  Returns None if there is no match.
         """
         if cls != Command:
-            return
+            raise Exception(f"from_frame not overridden in class {cls}")
 
-        for dc in cls._commands:
-            if dc._devicetype != devicetype:
-                continue
-            r = dc.from_frame(f)
+        # At the top level, we simply distinguish by frame size
+        subs = cls._framesizes.get(len(f), [])
+
+        for c in subs:
+            r = c.from_frame(f, devicetype=devicetype)
             if r:
                 return r
 
-        # At this point we can simply wrap the frame.  We don't know
-        # what kind of command this is (config, query, etc.) so we're
-        # unlikely ever to want to transmit it!
+        # The frame doesn't match any of the commands we know.
         return cls(f)
 
     @property
@@ -209,24 +241,33 @@ class Command(object):
         """The forward frame to be transmitted for this command."""
         return self._data
 
-    # XXX rename to send_twice ?
     @property
     def is_config(self):
         """Is this a configuration command?  (Does it need repeating to
         take effect?)
+
+        Use of this property is deprecated: access the "sendtwice"
+        attribute directly.
         """
-        return self._sendtwice
+        warnings.warn("Access 'sendtwice' directly instead of using 'is_config'",
+                      DeprecationWarning, stacklevel=2)
+        return self.sendtwice
 
     @property
     def is_query(self):
         """Does this command return a result?"""
-        return self._response is not None
+        return self.response is not None
 
     @property
-    def response(self):
+    def _response(self):
         """If this command returns a result, use this class for the response.
+
+        This property is provided for compatibility with old code.
+        Access the "response" attribute directly.
         """
-        return self._response
+        warnings.warn("Access 'response' directly instead of using '_response'",
+                      DeprecationWarning, stacklevel=2)
+        return self.response
 
     @staticmethod
     def _check_destination(destination):
